--[[ This file stores all of the data for (almost) every object in the game.
	Later on I may decide to streamline plants and animals into this mold
	and make them act like normal objects with special properties, such as
	behavioural patterns. I may attribute special behaioural properties to
	normal objects anyway, such as having fires burn combustibles or having
	machines do their functions, but that will be for later versions.

	Each objects stores its location in its own entry and the location itself
	will have an entry for the objects it holds, so I can use one to find the
	other.

	Every object also has an "integrity" stat which is essentially its hp.
	when the object's integrity reaches different points it may acquire new
	prefixes or become a new object or objects, such as a machine disassembling
	or a sword becoming dull or broken.

	Objects listed here are only the base objects. Every object in the game
	can also gain "prefixes" just like in Diablo 2, which give it special
	properties or describe its innate properties. For example, a basic object
	with be a "sword" or maybe even a "blade". From that it has prefixes,
	such as "iron" for what its made of, "long" or "short" which affect its
	reach, "fire enchanted" for a sort of magic property, "dull" which would
	be a temporary property that affects its damage, "rusted" which might be
	a special stage of deterioration that comes with other prefixes such as
	"dull", and other such things.

	Each object will also store two tables, one that holds all of the objects
	needed to construct it (if it can be contructed) and one that holds all of
	the objects that appear when it it destroyed. This second table may reference
	the first so it should come after it.

	Some objects will come in different colours. when this is the case, it will
	have two sets of images: one base set and one set that is monochrome. The
	monochrome set will be affected by the color_modulate variable that is set
	by a value in the object's relevant prefix. It will denote what color it is
	for use of either what dye it's been dyed or what metal it's made of or
	whatever.
--]]

--[[ Order of entries is as follows:
	BaseName: The name of the base object, minus prefixes. This entry is a table
			with with these values: {base name, base plural, {alternate names}, {plural names}}
	Image: The image to draw for it in a table: {world image, hud image, others}
	PhysicalStats: A table that has the following values:
				{Integrity, Weight(in grams), Volume(in cc), hardness}
--]]

-- So I don't confuse the term "object" with OOP I'll use the word "cosa"

-- table keep track of object related variables as well as a table ("live") which
-- stores all of the currently spawned objects on the current map

--[[Example instance of an object:
--]]

-- take actions for an object, based on its status. Used by map:Newday() and any object that acts in real time, as well as objects that are activated
function objects:Act(Index, Object, x, y)
	if Object.flags then
		if Object.flags.alive then
			--seed and grow plants
			if Object.flags.plant then
				AI:age(Object, x, y)
				AI:seed(Index, Object, x, y)
			end
			--grow seeds into plants
			if Object.flags.seed then
				AI:metamorphose(Index, Object, x, y)
			end
		end
	end
end

--for creating new instances of objects, takes the base plus it prefixes and other attributes as arguments
function objects:Instance(Location, Base, Variety, Custom, Status)
	local new_object = {  }
	setmetatable(new_object, Base)
	new_object.__index = Base
	local eachtable

	-- construct possibly empty tables so they exist, at least

	-- set base values
	for index, value in pairs(Base) do
		new_object[index] = value
	end

	--build flags table
	new_object.flags = {}

	-- get misc base stats
	for index, value in pairs(new_object.stats) do
		--set flags
		if index == "flags" then
			for index, value in pairs(new_object.stats[index]) do
				new_object.flags[value] = true
			end
		else
			new_object[index] = value
		end
	end

	--delete stats table to save memory
	new_object.stats = nil

	-- modify values unique types
	for index, value in pairs(Variety) do
		if type(Variety[index]) == "table" and index == not "actions" then
			local newtable = new_object[index]
			for i, v in ipairs(Variety[index]) do
				table.insert(newtable, v)
			end
			new_object[index] = newtable
			newtable = nil
		else
			new_object[index] = value
		end
	end

	-- Base information
	new_object.name = objects:NewName(Base.name, Variety.name)
	new_object.actions = objects:GetActions(Base.actions, Variety.actions)
	if type(Base.worldimage) == "table" then
		new_object.worldimage = Base.worldimage[Variety.image]
	end
		new_object.hudimage = Base.hudimage[Variety.image]
	new_object.status = {}
	new_object.Base = tostring(Base)
	new_object.Variety = tostring(Variety)
	new_object.inventory = {NODRAW = {}}

	-- set custom stats if set
	if Custom and type(Custom) == "table" then
		for i, v in pairs(Custom) do
			if i == "flags" then
				for sub_i, sub_v in pairs(Custom.flags) do
					new_object.flags[sub_v] = true
				end
			else
			new_object[i] = v
			end
		end
	end

	-- place object in the world
	if Location then

		if Base == objects.base.grass then table.insert(Location, 1, new_object)
		--elseif Base == objects.base.flower then table.insert(Location, 2, new_object)
		else table.insert(Location, new_object)
		end
	end

--	return new_object
end

-- Load list of actions avaliable for context menu
function objects:GetActions(Base, Variety, ...)
	local list = {}

		for i, v in ipairs(Base) do
			table.insert(list, v)
		end


	if Variety and Variety[1] then
		for i, v in ipairs(Variety) do
			--list[#list+1] = v
			table.insert(list, v)
		end
	end
	return list
end

function objects:NewName(Base, Variety,  ...)
-- Generates a new name
	local name = Variety.." "..Base[1]

	return name
end

-- Main Functions

function objects:load()
--TEST
objects:Instance( player.inventory[2][4], objects.base.seeds, objects.variety.wild_lupin)
objects:Instance( player.inventory[1][5], objects.base.seeds, objects.variety.Field_Poppy)
objects:Instance( player.inventory[3][3], objects.base.seeds, objects.variety.Red_Maple)
objects:Instance( player.inventory[4][2], objects.base.seeds, objects.variety.meadow_foxtail)

objects:Instance( map.tile_map[50][50].objects, objects.base.grass, objects.variety.meadow_foxtail, {age = 50, age_step = 5})
objects:Instance( map.tile_map[51][50].objects, objects.base.grass, objects.variety.meadow_foxtail)
objects:Instance( map.tile_map[50][51].objects, objects.base.grass, objects.variety.meadow_foxtail)
objects:Instance( map.tile_map[51][51].objects, objects.base.grass, objects.variety.meadow_foxtail)
objects:Instance( map.tile_map[51][52].objects, objects.base.flower, objects.variety.wild_lupin)
end

function objects:update()

end

function objects:draw()

end
